/*----------------------------------------------------------------------
  File    : random.c
  Contents: generate random numbers
  Author  : Christian Borgelt
  History : 19.11.1997 file created
----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/

int main (int argc, char *argv[])
{                               /* --- generate random numbers */
  int    n;                     /* number of numbers to generate */
  double min, max;              /* interval to generate numbers in */
  double f;                     /* range factor for rand() */
  char   *fmt = "%g";           /* output format */

  if ((argc != 4) && (argc != 5)) { /* if wrong number of args. given */
    printf("usage: %s n min max [fmt]\n", argv[0]);
    printf("generate n random numbers in the interval [min, max]\n");
    printf("the output format fmt is optional, default: %%g\n");
    return 0;                   /* print a usage message */
  }                             /* and abort the program */
  n   = atoi(argv[1]);          /* get number of random numbers */
  min = atof(argv[2]);          /* and the interval to generate */
  max = atof(argv[3]);          /* the random numbers in */
  if ((n < 0) || (max < min)) { /* check for correct arguments */
    printf("%s: illegal arguments\n", argv[0]); return 1; }
  if (argc > 4) fmt = argv[4];  /* if present, get output format */
  srand((unsigned)time(NULL));  /* init. random number generator */
  f = (max-min) /RAND_MAX;      /* compute range factor for rand() */
  while (--n >= 0) {            /* generate n random numbers */
    printf(fmt, rand() *f +min); putchar('\n'); }
  return 0;                     /* return 'ok' */
}  /* main() */

/*----------------------------------------------------------------------
The above program generates n random numbers in the interval [min, max]
where n, min, and max have to be given as command line arguments. Random
numbers, or, more precisely, pseudo-random numbers (since they are not
really random), can be generated with the standard library function
`rand'. On each call this function returns a new random integer number
in the interval [0, RAND_MAX], where RAND_MAX is a preprocessor constant
defined in `stdlib.h'. It is advisable to initialize the pseudo-random
number generator used by the function `rand' with a start value by
calling the function `srand' (seed random numbers). The start value is
not the first number returned by `rand' but just a number used in the
computations that yield a new pseudo-random number. If the pseudo-random
number generator is not initialized, the start value is automatically
chosen to be 1. Since the numbers generated are not really random, but
are computed using some strange formula, the pseudo-random number
generator always yields the same sequence of number, if initialized
with the same start value. Hence, if not initialized, the pseudo-random
number generator always yields the sequence of numbers that corresponds
to start value 1. To avoid this, we initialize the pseudo-random number
generator with the current time, measured in seconds since January 1st
1970. This is the standard UNIX time format returned by the function
`time', which is defined in the header file `time.h'. To generate a
random number in the interval [min, max] we transform the pseudo-random
number returned by `rand' (which lies, see above, in the interval
[0, RAND_MAX]) by multiplying it by (max-min)/RAND_MAX and adding min.
----------------------------------------------------------------------*/
